/* Class autosprintf - formatted output to an ostream.
   Copyright (C) 2002, 2012-2016 Free Software Foundation, Inc.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation; either version 2.1 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

#ifndef _AUTOSPRINTF_H
#define _AUTOSPRINTF_H

/* This feature is available in gcc versions 2.5 and later.  */
#if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 5) || __STRICT_ANSI__
# define _AUTOSPRINTF_ATTRIBUTE_FORMAT() /* empty */
#else
/* The __-protected variants of 'format' and 'printf' attributes
   are accepted by gcc versions 2.6.4 (effectively 2.7) and later.  */
# if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 7)
#  define _AUTOSPRINTF_ATTRIBUTE_FORMAT() \
  __attribute__ ((__format__ (__printf__, 2, 3)))
# else
#  define _AUTOSPRINTF_ATTRIBUTE_FORMAT() \
  __attribute__ ((format (printf, 2, 3)))
# endif
#endif

#include <string>
#include <iostream>

#ifndef DLLX_EXPORT
# if defined(_MSC_VER) || defined(_WIN32) || defined(_DECLSPEC_SUPPORTED)
#  define DLLX_EXPORT __declspec(dllexport)
#  define DLLX_IMPORT __declspec(dllimport)
# elif HAVE_VISIBILITY
#  define DLLX_EXPORT __attribute__((__visibility__("default")))
#  define DLLX_IMPORT
# else
#  define DLLX_EXPORT
#  define DLLX_IMPORT
# endif
#endif

#ifndef LIBASPRINTF_EXPORT
# if defined(IN_LIBASPRINTF)
#  define LIBASPRINTF_EXPORT DLLX_EXPORT
# else
#  define LIBASPRINTF_EXPORT DLLX_IMPORT
# endif
#endif


namespace gnu
{
  /* A temporary object, usually allocated on the stack, representing
     the result of an asprintf() call.  */
  class autosprintf
  {
  public:
    /* Constructor: takes a format string and the printf arguments.  */
    LIBASPRINTF_EXPORT autosprintf (const char *format, ...)
                _AUTOSPRINTF_ATTRIBUTE_FORMAT();
    /* Copy constructor.  */
    LIBASPRINTF_EXPORT autosprintf (const autosprintf& src);
    LIBASPRINTF_EXPORT autosprintf& operator = (autosprintf copy);
    /* Destructor: frees the temporarily allocated string.  */
    LIBASPRINTF_EXPORT ~autosprintf ();
    /* Conversion to string.  */
    LIBASPRINTF_EXPORT operator char * () const;
    LIBASPRINTF_EXPORT operator std::string () const;
    /* Output to an ostream.  */
    friend inline std::ostream& operator<< (std::ostream& stream, const autosprintf& tmp)
    {
      stream << (tmp.str ? tmp.str : "(error in autosprintf)");
      return stream;
    }
  private:
    char *str;
  };
}

#endif /* _AUTOSPRINTF_H */
